@@ -56,6 +56,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.StringJoiner;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.function.Function;
 import java.util.function.Supplier;
 import org.apache.fury.Fury;
@@ -157,6 +158,9 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
     jitCallbackUpdateFields = new HashMap<>();
   }
 
+  // Must be static to be shared across the whole process life.
+  private static final Map<String, Map<String, Integer>> idGenerator = new ConcurrentHashMap<>();
+
   public String codecClassName(Class<?> beanClass) {
     String name = ReflectionUtils.getClassNameWithoutPackage(beanClass).replace("$", "_");
     StringBuilder nameBuilder = new StringBuilder(name);
@@ -167,12 +171,17 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
     } else {
       nameBuilder.append("Fury");
     }
-    nameBuilder.append(codecSuffix()).append("Codec");
-    nameBuilder.append('_').append(fury.getConfig().getConfigHash());
-    String classUniqueId = CodeGenerator.getClassUniqueId(beanClass);
-    if (StringUtils.isNotBlank(classUniqueId)) {
-      nameBuilder.append('_').append(classUniqueId);
+    nameBuilder.append("Codec").append(codecSuffix());
+    Map<String, Integer> subGenerator =
+        idGenerator.computeIfAbsent(nameBuilder.toString(), k -> new ConcurrentHashMap<>());
+    String key = fury.getConfig().getConfigHash() + "_" + CodeGenerator.getClassUniqueId(beanClass);
+    Integer id = subGenerator.get(key);
+    if (id == null) {
+      synchronized (subGenerator) {
+        id = subGenerator.computeIfAbsent(key, k -> subGenerator.size());
+      }
     }
+    nameBuilder.append('_').append(id);
     return nameBuilder.toString();
   }
 
@@ -1269,18 +1278,16 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
         readBuilder.add(
             readContainerElements(
                 elementType, true, elemSerializer, null, buffer, collection, size));
-        Set<Expression> cutPoint = ofHashSet(buffer, collection, size);
-        if (maybeDecl) { // For `isDeclType`
-          cutPoint.add(flags);
-        }
-        Expression sameElementClassRead =
-            invokeGenerated(ctx, cutPoint, readBuilder, "sameElementClassRead", false);
         // Same element class read end
-        action =
-            new If(
-                sameElementClass,
-                sameElementClassRead,
-                readContainerElements(elementType, true, null, null, buffer, collection, size));
+        Set<Expression> cutPoint = ofHashSet(buffer, collection, size);
+        Expression differentElemTypeRead =
+            invokeGenerated(
+                ctx,
+                cutPoint,
+                readContainerElements(elementType, true, null, null, buffer, collection, size),
+                "differentTypeElemsRead",
+                false);
+        action = new If(sameElementClass, readBuilder, differentElemTypeRead);
       } else {
         Literal hasNullFlag = Literal.ofInt(CollectionFlags.HAS_NULL);
         Expression hasNull = eq(new BitAnd(flags, hasNullFlag), hasNullFlag, "hasNull");
@@ -1290,18 +1297,16 @@ public abstract class BaseObjectCodecBuilder extends CodecBuilder {
         readBuilder.add(
             readContainerElements(
                 elementType, false, elemSerializer, hasNull, buffer, collection, size));
-        Set<Expression> cutPoint = ofHashSet(buffer, collection, size, hasNull);
-        if (maybeDecl) { // For `isDeclType`
-          cutPoint.add(flags);
-        }
         // Same element class read end
-        Expression sameElementClassRead =
-            invokeGenerated(ctx, cutPoint, readBuilder, "sameElementClassRead", false);
-        action =
-            new If(
-                sameElementClass,
-                sameElementClassRead,
-                readContainerElements(elementType, false, null, hasNull, buffer, collection, size));
+        Set<Expression> cutPoint = ofHashSet(buffer, collection, size, hasNull);
+        Expression differentTypeElemsRead =
+            invokeGenerated(
+                ctx,
+                cutPoint,
+                readContainerElements(elementType, false, null, hasNull, buffer, collection, size),
+                "differentTypeElemsRead",
+                false);
+        action = new If(sameElementClass, readBuilder, differentTypeElemsRead);
       }
       builder.add(action);
     }
