@@ -30,6 +30,7 @@ import org.apache.fury.memory.Platform;
 import org.apache.fury.meta.DeflaterMetaCompressor;
 import org.apache.fury.meta.MetaCompressor;
 import org.apache.fury.pool.ThreadPoolFury;
+import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.serializer.JavaSerializer;
 import org.apache.fury.serializer.ObjectStreamSerializer;
@@ -56,10 +57,11 @@ public final class FuryBuilder {
     ENABLE_CLASS_REGISTRATION_FORCIBLY = "true".equals(flagValue) || "1".equals(flagValue);
   }
 
+  String name;
   boolean checkClassVersion = false;
   Language language = Language.JAVA;
   boolean trackingRef = false;
-  boolean copyRef = true;
+  boolean copyRef = false;
   boolean basicTypesRefIgnored = true;
   boolean stringRefIgnored = true;
   boolean timeRefIgnored = true;
@@ -315,6 +317,22 @@ public final class FuryBuilder {
   /** Whether enable scala-specific serialization optimization. */
   public FuryBuilder withScalaOptimizationEnabled(boolean enableScalaOptimization) {
     this.scalaOptimizationEnabled = enableScalaOptimization;
+    if (enableScalaOptimization) {
+      try {
+        Class.forName(
+            ReflectionUtils.getPackage(Fury.class) + ".serializer.scala.ScalaSerializers");
+      } catch (ClassNotFoundException e) {
+        LOG.warn(
+            "`fury-scala` library is not in the classpath, please add it to class path and invoke "
+                + "`org.apache.fury.serializer.scala.ScalaSerializers.registerSerializers` for peek performance");
+      }
+    }
+    return this;
+  }
+
+  /** Set name for Fury serialization. */
+  public FuryBuilder withName(String name) {
+    this.name = name;
     return this;
   }
 
