@@ -139,6 +139,7 @@ import org.apache.fury.serializer.collection.GuavaCollectionSerializers;
 import org.apache.fury.serializer.collection.ImmutableCollectionSerializers;
 import org.apache.fury.serializer.collection.MapSerializer;
 import org.apache.fury.serializer.collection.MapSerializers;
+import org.apache.fury.serializer.collection.SubListSerializers;
 import org.apache.fury.serializer.collection.SynchronizedSerializers;
 import org.apache.fury.serializer.collection.UnmodifiableSerializers;
 import org.apache.fury.serializer.scala.SingletonCollectionSerializer;
@@ -266,6 +267,7 @@ public class ClassResolver {
     extRegistry = new ExtRegistry();
     extRegistry.objectGenericType = buildGenericType(OBJECT_TYPE);
     shimDispatcher = new ShimDispatcher(fury);
+    ClassResolver._addGraalvmClassRegistry(fury.getConfig().getConfigHash(), this);
   }
 
   public void initialize() {
@@ -336,6 +338,7 @@ public class ClassResolver {
     SynchronizedSerializers.registerSerializers(fury);
     UnmodifiableSerializers.registerSerializers(fury);
     ImmutableCollectionSerializers.registerSerializers(fury);
+    SubListSerializers.registerSerializers(fury, true);
     if (fury.getConfig().registerGuavaTypes()) {
       GuavaCollectionSerializers.registerDefaultSerializers(fury);
     }
@@ -408,6 +411,10 @@ public class ClassResolver {
     }
   }
 
+  public void register(String className) {
+    register(loadClass(className, false, 0, false));
+  }
+
   public void register(Class<?>... classes) {
     for (Class<?> cls : classes) {
       register(cls);
@@ -417,14 +424,7 @@ public class ClassResolver {
   public void register(Class<?> cls, boolean createSerializer) {
     register(cls);
     if (createSerializer) {
-      ClassInfo classInfo = getClassInfo(cls);
-      if (metaContextShareEnabled && needToWriteClassDef(classInfo.serializer)) {
-        ClassDef classDef = classInfo.classDef;
-        if (classDef == null) {
-          classDef = buildClassDef(classInfo);
-        }
-        buildMetaSharedClassInfo(Tuple2.of(classDef, null), classDef);
-      }
+      createSerializerAhead(cls);
     }
   }
 
@@ -487,13 +487,21 @@ public class ClassResolver {
     extRegistry.classIdGenerator++;
   }
 
+  public void register(String className, int classId) {
+    register(loadClass(className, false, 0, false), classId);
+  }
+
   public void register(Class<?> cls, Short id, boolean createSerializer) {
     register(cls, id);
     if (createSerializer) {
-      getSerializer(cls);
+      createSerializerAhead(cls);
     }
   }
 
+  public void register(String className, Short classId, boolean createSerializer) {
+    register(loadClass(className, false, 0, false), classId, createSerializer);
+  }
+
   public boolean isRegistered(Class<?> cls) {
     return extRegistry.registeredClassIdMap.get(cls) != null;
   }
@@ -699,7 +707,7 @@ public class ClassResolver {
 
   /**
    * Set serializer to avoid circular error when there is a serializer query for fields by {@link
-   * #buildMetaSharedClassInfo} and {@link #getSerializer(Class)} which access current creating
+   * #readClassInfoWithMetaShare} and {@link #getSerializer(Class)} which access current creating
    * serializer. This method is used to avoid overwriting existing serializer for class when
    * creating a data serializer for serialization of parts fields of a class.
    */
@@ -824,6 +832,12 @@ public class ClassResolver {
       // serialized, which will create a class info with serializer null, see `#writeClassInternal`
       return classInfo.serializer.getClass();
     } else {
+      if (getSerializerFactory() != null) {
+        Serializer serializer = getSerializerFactory().createSerializer(fury, cls);
+        if (serializer != null) {
+          return serializer.getClass();
+        }
+      }
       if (NonexistentClass.isNonexistent(cls)) {
         return NonexistentClassSerializers.getSerializer(fury, "Unknown", cls).getClass();
       }
@@ -841,7 +855,11 @@ public class ClassResolver {
       } else if (Functions.isLambda(cls)) {
         return LambdaSerializer.class;
       } else if (ReflectionUtils.isJdkProxy(cls)) {
-        return JdkProxySerializer.class;
+        if (JavaSerializer.getWriteReplaceMethod(cls) != null) {
+          return ReplaceResolveSerializer.class;
+        } else {
+          return JdkProxySerializer.class;
+        }
       } else if (Calendar.class.isAssignableFrom(cls)) {
         return TimeSerializers.CalendarSerializer.class;
       } else if (ZoneId.class.isAssignableFrom(cls)) {
@@ -1207,6 +1225,32 @@ public class ClassResolver {
     return serializer;
   }
 
+  private void createSerializerAhead(Class<?> cls) {
+    ClassInfo classInfo = getClassInfo(cls);
+    ClassInfo deserializationClassInfo;
+    if (metaContextShareEnabled && needToWriteClassDef(classInfo.serializer)) {
+      ClassDef classDef = classInfo.classDef;
+      if (classDef == null) {
+        classDef = buildClassDef(classInfo);
+      }
+      deserializationClassInfo = buildMetaSharedClassInfo(Tuple2.of(classDef, null), classDef);
+      if (deserializationClassInfo != null && GraalvmSupport.isGraalBuildtime()) {
+        getGraalvmClassRegistry()
+            .deserializerClassMap
+            .put(classDef.getId(), deserializationClassInfo.serializer.getClass());
+        Tuple2<ClassDef, ClassInfo> classDefTuple = extRegistry.classIdToDef.get(classDef.getId());
+        // empty serializer for graalvm build time
+        classDefTuple.f1.serializer = null;
+        extRegistry.classIdToDef.put(classDef.getId(), Tuple2.of(classDefTuple.f0, null));
+      }
+    }
+    if (GraalvmSupport.isGraalBuildtime()) {
+      // Instance for generated class should be hold at graalvm runtime only.
+      getGraalvmClassRegistry().serializerClassMap.put(cls, classInfo.serializer.getClass());
+      classInfo.serializer = null;
+    }
+  }
+
   private String generateSecurityMsg(Class<?> cls) {
     String tpl =
         "%s is not registered, please check whether it's the type you want to serialize or "
@@ -1344,22 +1388,26 @@ public class ClassResolver {
     if ((header & 0b1) == 0) {
       return getOrUpdateClassInfo((short) id);
     }
-    ObjectArray<ClassInfo> readClassInfos = metaContext.readClassInfos;
-    ClassInfo classInfo = readClassInfos.get(id);
+    ClassInfo classInfo = metaContext.readClassInfos.get(id);
     if (classInfo == null) {
-      ObjectArray<ClassDef> readClassDefs = metaContext.readClassDefs;
-      ClassDef classDef = readClassDefs.get(id);
-      Tuple2<ClassDef, ClassInfo> classDefTuple = extRegistry.classIdToDef.get(classDef.getId());
-      if (classDefTuple == null || classDefTuple.f1 == null) {
-        classInfo = buildMetaSharedClassInfo(classDefTuple, classDef);
-      } else {
-        classInfo = classDefTuple.f1;
-      }
-      readClassInfos.set(id, classInfo);
+      classInfo = readClassInfoWithMetaShare(metaContext, id);
     }
     return classInfo;
   }
 
+  private ClassInfo readClassInfoWithMetaShare(MetaContext metaContext, int index) {
+    ClassDef classDef = metaContext.readClassDefs.get(index);
+    Tuple2<ClassDef, ClassInfo> classDefTuple = extRegistry.classIdToDef.get(classDef.getId());
+    ClassInfo classInfo;
+    if (classDefTuple == null || classDefTuple.f1 == null || classDefTuple.f1.serializer == null) {
+      classInfo = buildMetaSharedClassInfo(classDefTuple, classDef);
+    } else {
+      classInfo = classDefTuple.f1;
+    }
+    metaContext.readClassInfos.set(index, classInfo);
+    return classInfo;
+  }
+
   private ClassInfo buildMetaSharedClassInfo(
       Tuple2<ClassDef, ClassInfo> classDefTuple, ClassDef classDef) {
     ClassInfo classInfo;
@@ -1471,7 +1519,7 @@ public class ClassResolver {
   public void readClassDefs(MemoryBuffer buffer) {
     MetaContext metaContext = fury.getSerializationContext().getMetaContext();
     assert metaContext != null : SET_META__CONTEXT_MSG;
-    int numClassDefs = buffer.readVarUint32Small14();
+    int numClassDefs = buffer.readVarUint32Small7();
     for (int i = 0; i < numClassDefs; i++) {
       long id = buffer.readInt64();
       Tuple2<ClassDef, ClassInfo> tuple2 = extRegistry.classIdToDef.get(id);
@@ -1485,9 +1533,7 @@ public class ClassResolver {
         tuple2 = readClassDef(buffer, id);
       }
       metaContext.readClassDefs.add(tuple2.f0);
-      // Will be set lazily, so even some classes doesn't exist, remaining classinfo
-      // can be created still.
-      metaContext.readClassInfos.add(null);
+      metaContext.readClassInfos.add(tuple2.f1);
     }
   }
 
@@ -1779,6 +1825,11 @@ public class ClassResolver {
   }
 
   private Class<?> loadClass(String className, boolean isEnum, int arrayDims) {
+    return loadClass(className, isEnum, arrayDims, fury.getConfig().deserializeNonexistentClass());
+  }
+
+  private Class<?> loadClass(
+      String className, boolean isEnum, int arrayDims, boolean deserializeNonexistentClass) {
     extRegistry.classChecker.checkClass(this, className);
     try {
       return Class.forName(className, false, fury.getClassLoader());
@@ -1790,7 +1841,7 @@ public class ClassResolver {
             String.format(
                 "Class %s not found from classloaders [%s, %s]",
                 className, fury.getClassLoader(), Thread.currentThread().getContextClassLoader());
-        if (fury.getConfig().deserializeNonexistentClass()) {
+        if (deserializeNonexistentClass) {
           LOG.warn(msg);
           return NonexistentClass.getNonexistentClass(
               className, isEnum, arrayDims, metaContextShareEnabled);
@@ -1909,33 +1960,55 @@ public class ClassResolver {
     return fury;
   }
 
-  private static final ConcurrentMap<Integer, List<ClassResolver>> GRAALVM_REGISTRY =
+  private static final ConcurrentMap<Integer, GraalvmClassRegistry> GRAALVM_REGISTRY =
       new ConcurrentHashMap<>();
 
   // CHECKSTYLE.OFF:MethodName
   public static void _addGraalvmClassRegistry(int furyConfigHash, ClassResolver classResolver) {
     // CHECKSTYLE.ON:MethodName
     if (GraalvmSupport.isGraalBuildtime()) {
-      List<ClassResolver> resolvers =
-          GRAALVM_REGISTRY.computeIfAbsent(
-              furyConfigHash, k -> Collections.synchronizedList(new ArrayList<>()));
-      resolvers.add(classResolver);
+      GraalvmClassRegistry registry =
+          GRAALVM_REGISTRY.computeIfAbsent(furyConfigHash, k -> new GraalvmClassRegistry());
+      registry.resolvers.add(classResolver);
     }
   }
 
+  private static class GraalvmClassRegistry {
+    private final List<ClassResolver> resolvers;
+    private final Map<Class<?>, Class<? extends Serializer>> serializerClassMap;
+    private final Map<Long, Class<? extends Serializer>> deserializerClassMap;
+
+    private GraalvmClassRegistry() {
+      resolvers = Collections.synchronizedList(new ArrayList<>());
+      serializerClassMap = new ConcurrentHashMap<>();
+      deserializerClassMap = new ConcurrentHashMap<>();
+    }
+  }
+
+  private GraalvmClassRegistry getGraalvmClassRegistry() {
+    return GRAALVM_REGISTRY.computeIfAbsent(
+        fury.getConfig().getConfigHash(), k -> new GraalvmClassRegistry());
+  }
+
   private Class<? extends Serializer> getSerializerClassFromGraalvmRegistry(Class<?> cls) {
-    List<ClassResolver> classResolvers = GRAALVM_REGISTRY.get(fury.getConfig().getConfigHash());
-    if (classResolvers == null || classResolvers.isEmpty()) {
+    GraalvmClassRegistry registry = getGraalvmClassRegistry();
+    List<ClassResolver> classResolvers = registry.resolvers;
+    if (classResolvers.isEmpty()) {
       return null;
     }
     for (ClassResolver classResolver : classResolvers) {
       if (classResolver != this) {
         ClassInfo classInfo = classResolver.classInfoMap.get(cls);
-        if (classInfo != null) {
+        if (classInfo != null && classInfo.serializer != null) {
           return classInfo.serializer.getClass();
         }
       }
     }
+    Class<? extends Serializer> serializerClass = registry.serializerClassMap.get(cls);
+    // noinspection Duplicates
+    if (serializerClass != null) {
+      return serializerClass;
+    }
     if (GraalvmSupport.isGraalRuntime()) {
       if (Functions.isLambda(cls) || ReflectionUtils.isJdkProxy(cls)) {
         return null;
@@ -1947,18 +2020,16 @@ public class ClassResolver {
 
   private Class<? extends Serializer> getMetaSharedDeserializerClassFromGraalvmRegistry(
       Class<?> cls, ClassDef classDef) {
-    List<ClassResolver> classResolvers = GRAALVM_REGISTRY.get(fury.getConfig().getConfigHash());
-    if (classResolvers == null || classResolvers.isEmpty()) {
+    GraalvmClassRegistry registry = getGraalvmClassRegistry();
+    List<ClassResolver> classResolvers = registry.resolvers;
+    if (classResolvers.isEmpty()) {
       return null;
     }
-    for (ClassResolver classResolver : classResolvers) {
-      if (classResolver != this) {
-        Tuple2<ClassDef, ClassInfo> tuple2 =
-            classResolver.extRegistry.classIdToDef.get(classDef.getId());
-        if (tuple2 != null && tuple2.f1 != null) {
-          return tuple2.f1.serializer.getClass();
-        }
-      }
+    Class<? extends Serializer> deserializerClass =
+        registry.deserializerClassMap.get(classDef.getId());
+    // noinspection Duplicates
+    if (deserializerClass != null) {
+      return deserializerClass;
     }
     if (GraalvmSupport.isGraalRuntime()) {
       if (Functions.isLambda(cls) || ReflectionUtils.isJdkProxy(cls)) {
